// Test de la capacité d'une batterie d'accumulateurs
// P. Pénard 01 2017
// Mesure automatique sous contrôle d'un relais
// Résolution (approximation) de l'intégrale de la capacité sous la forme d'une somme de Riemann
// On fait le minimum de calcul dans la boucle de mesure de manière à ce que les 
// échantillons soient le plus possible équidistants (en temps ici). 

#define RESA 10.1  // Valeur de la résistance aux bornes de laquelle on fait la mesure (en Ohms)
#define RESB 16.5  // Valeur de la résistance série (en Ohms)
//#define VMIN 7.5  // (3 S Li-Ion) Tension minimale de l'accu en dessous de laquelle le test s'arrête (en Volts)
#define VMIN 9  // (3 S Lipo)
#define PERMES 10000 // Période des mesures (en millisecondes)

// Les pins
const int Mesure = A0; // Mesure de la tension
const int Cde_relais = 2; // Commande du relais

// Variables
int valeur_lue = 0;
int valeur_min = 0;
double Courant = 0.0;
double capacite = 0.0;
boolean mesure = true;
int cpt_aff = 0;
unsigned long temps = 0;
unsigned long temps_prec;

// La fonction d'init
void setup() {
  valeur_min = VMIN * (RESA /(RESA + RESB)) * (1023 / 5); // Calculée ici pour ne pas avoir à le faire dans la boucle de mesur
  Serial.begin(9600);
  pinMode(Cde_relais, OUTPUT);
  digitalWrite(Cde_relais, HIGH);
  delay(1000);
  Serial.println("Mesure de la capacite d'une batterie");
  Serial.print("Tension de fin de test = ");
  Serial.print(VMIN);
  Serial.println(" Volts");
  Serial.println();
}

// Transmission série
void transmettre(){ 
  if (cpt_aff == 12) {
      Serial.print("Courant = " );
      Serial.print(Courant, 4);
      Serial.print(" A;  " );
      Serial.print("Capacite = " );
      Serial.print(capacite, 4);
      Serial.println(" mA.h"); 
      cpt_aff = 0;
  }
  else cpt_aff++;
}

// La boucle de mesure
void loop() {
  // On pourrait penser que (temps - temps_prec) fait doublon avec delay(PERMES) mais ce n'est pas le cas ;o)
  // En fait on ne sait pas combien de temps va prendre le reste de la boucle et en particulier la transmission série
  temps_prec = temps;
  temps = millis();
  valeur_lue = analogRead(Mesure);
  if (mesure){
    if (valeur_lue > valeur_min){
      Courant = ((double)valeur_lue * 5 / 1023) / RESA;
      capacite += Courant * (temps - temps_prec) / 3600;
      transmettre();
      delay(PERMES);
    }
    else{      
      digitalWrite(Cde_relais, LOW);
      Serial.println(" ");
      Serial.println("Fin de la mesure");
      mesure = false;
    }
  }
}
